@@ -34,15 +34,21 @@ typedef struct {
 
 /* Evince stock icons */
 static const EvStockIcon stock_icons [] = {
+	{ EV_STOCK_ZOOM,	     "zoom" },
 	{ EV_STOCK_ZOOM_PAGE,        "zoom-fit-page" },
 	{ EV_STOCK_ZOOM_WIDTH,       "zoom-fit-width" },
-	{ EV_STOCK_LEAVE_FULLSCREEN, "view-restore" },
 	{ EV_STOCK_VIEW_DUAL,        "view-page-facing" },
 	{ EV_STOCK_VIEW_CONTINUOUS,  "view-page-continuous" },
 	{ EV_STOCK_ROTATE_LEFT,      "object-rotate-left"},
 	{ EV_STOCK_ROTATE_RIGHT,     "object-rotate-right"},
+	{ EV_STOCK_RUN_PRESENTATION, "x-office-presentation"},
 };
 
+/**
+ * ev_stock_icons_init:
+ *
+ * Creates a new icon factory, adding the base stock icons to it.
+ */
 void
 ev_stock_icons_init (void)
 {
